'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _objType = require('./objType');

var _objType2 = _interopRequireDefault(_objType);

var _JSONObjectNode = require('./JSONObjectNode');

var _JSONObjectNode2 = _interopRequireDefault(_JSONObjectNode);

var _JSONArrayNode = require('./JSONArrayNode');

var _JSONArrayNode2 = _interopRequireDefault(_JSONArrayNode);

var _JSONIterableNode = require('./JSONIterableNode');

var _JSONIterableNode2 = _interopRequireDefault(_JSONIterableNode);

var _JSONValueNode = require('./JSONValueNode');

var _JSONValueNode2 = _interopRequireDefault(_JSONValueNode);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

var typeMap = {
  Object: function Object(props) {
    return _react2.default.createElement(_JSONObjectNode2.default, props);
  },
  Error: function Error(props) {
    return _react2.default.createElement(_JSONObjectNode2.default, props);
  },
  WeakMap: function WeakMap(props) {
    return _react2.default.createElement(_JSONObjectNode2.default, props);
  },
  WeakSet: function WeakSet(props) {
    return _react2.default.createElement(_JSONObjectNode2.default, props);
  },

  Array: function Array(props) {
    return _react2.default.createElement(_JSONArrayNode2.default, props);
  },

  Iterable: function Iterable(props) {
    return _react2.default.createElement(_JSONIterableNode2.default, props);
  },
  Map: function Map(props) {
    return _react2.default.createElement(_JSONIterableNode2.default, props);
  },
  Set: function Set(props) {
    return _react2.default.createElement(_JSONIterableNode2.default, props);
  },

  Number: function Number(props) {
    return _react2.default.createElement(_JSONValueNode2.default, props);
  },
  Custom: function Custom(props) {
    return _react2.default.createElement(_JSONValueNode2.default, props);
  },

  String: function String(props) {
    return _react2.default.createElement(_JSONValueNode2.default, _extends({}, props, { valueGetter: function valueGetter(raw) {
        return '"' + raw + '"';
      } }));
  },
  Boolean: function Boolean(props) {
    return _react2.default.createElement(_JSONValueNode2.default, _extends({}, props, { valueGetter: function valueGetter(raw) {
        return raw ? 'true' : 'false';
      } }));
  },
  Date: function Date(props) {
    return _react2.default.createElement(_JSONValueNode2.default, _extends({}, props, { valueGetter: function valueGetter(raw) {
        return raw.toISOString();
      } }));
  },
  Null: function Null(props) {
    return _react2.default.createElement(_JSONValueNode2.default, _extends({}, props, { valueGetter: function valueGetter() {
        return 'null';
      } }));
  },
  Undefined: function Undefined(props) {
    return _react2.default.createElement(_JSONValueNode2.default, _extends({}, props, { valueGetter: function valueGetter() {
        return 'undefined';
      } }));
  },

  Function: function Function(props) {
    return _react2.default.createElement(_JSONValueNode2.default, _extends({}, props, { valueGetter: function valueGetter(raw) {
        return raw.toString();
      } }));
  },
  Symbol: function Symbol(props) {
    return _react2.default.createElement(_JSONValueNode2.default, _extends({}, props, { valueGetter: function valueGetter(raw) {
        return raw.toString();
      } }));
  }
};

var nestedTypes = {
  Object: true,
  Error: true,
  WeakMap: true,
  WeakSet: true,
  Array: true,
  Iterable: true,
  Map: true,
  Set: true
};

var JSONNode = function JSONNode(_ref) {
  var getItemString = _ref.getItemString,
      keyPath = _ref.keyPath,
      labelRenderer = _ref.labelRenderer,
      styling = _ref.styling,
      value = _ref.value,
      valueRenderer = _ref.valueRenderer,
      isCustomNode = _ref.isCustomNode,
      onChange = _ref.onChange,
      rest = _objectWithoutProperties(_ref, ['getItemString', 'keyPath', 'labelRenderer', 'styling', 'value', 'valueRenderer', 'isCustomNode', 'onChange']);

  var nodeType = isCustomNode(value) ? 'Custom' : (0, _objType2.default)(value);

  var simpleNodeProps = {
    getItemString: getItemString,
    key: keyPath[0],
    keyPath: keyPath,
    labelRenderer: labelRenderer,
    nodeType: nodeType,
    styling: styling,
    value: value,
    valueRenderer: valueRenderer,
    onChange: onChange
  };

  var nestedNodeProps = _extends({}, rest, simpleNodeProps, {
    data: value,
    isCustomNode: isCustomNode,
    onChange: onChange
  });

  var handler = typeMap[nodeType];
  if (!handler) {
    return null;
  }

  var isNested = nestedTypes[nodeType];
  return handler(isNested ? nestedNodeProps : simpleNodeProps);
};

JSONNode.propTypes = {
  getItemString: _propTypes2.default.func.isRequired,
  keyPath: _propTypes2.default.arrayOf(_propTypes2.default.oneOfType([_propTypes2.default.string, _propTypes2.default.number])).isRequired,
  labelRenderer: _propTypes2.default.func.isRequired,
  styling: _propTypes2.default.func.isRequired,
  value: _propTypes2.default.any,
  valueRenderer: _propTypes2.default.func.isRequired,
  isCustomNode: _propTypes2.default.func.isRequired,
  onChange: _propTypes2.default.func.isRequired
};

exports.default = JSONNode;